###################################################################################################
#########################   Differential Expression Analysis using edgeR  #########################
###################################################################################################

#2.1 Reading in the Data
library(edgeR)
load("/Users/rsonmez/Desktop/teaching/bioinformatics-core-shared-training-cruk-bioinf-sschool-24d7cad/Day3/Counts.RData")
Counts <- tmp$counts
colnames(Counts) <- c("16N", "16T", "18N", "18T", "19N", "19T") # sample names #N=normal; T=tumour
dim(Counts) # genes by samples matrix
head(Counts) # to see counts of top 6 genes in the matrix
Counts[1:20,] # to see counts of top 20 genes in the matrix

#2.2 Creating a DGEList object
dgList <- DGEList(counts=Counts, genes=rownames(Counts))
dgList
dgList$samples # shows the library size of each sample. Equal to simple column sums of matrix. e.g  sum(Counts[,1])
head(dgList$counts) 
head(dgList$genes) 

#2.3 Filtering
#CPM (counts per million) or RPKM values are useful descriptive measures for the expression level of a gene
countsPerMillion <- cpm(dgList) #corrects for library size of each sample, therefore makes genes become comparable between samples
summary(countsPerMillion)

countCheck <- countsPerMillion > 1 #logical indexing
head(countCheck)
keep <- which(rowSums(countCheck) >= 2) #we retain only those genes that are represented at least 1 cpm reads in at least two samples 
dgList <- dgList[keep,] 
summary(cpm(dgList)) #compare this to the original summary

#2.4 Normalisation
#TMM (trimmed mean of M values) normalization method to calculate effective libraries sizes.
#A scaling factor is calculated. Original library size x  Scaling factor = Effective library size. 
#A normalization factor below one indicates that a small number of high count genes are monopolizing the sequencing,
#causing the counts for other genes to be lower than would be usual given the library size. 
#As a result, the library size will be scaled down, analogous to scaling the counts upwards in that library. 
#Conversely, a factor above one scales up the library size, analogous to downscaling the counts.
?calcNormFactors
dgList <- calcNormFactors(dgList, method="TMM") 
dgList$samples

#2.5 Data Exploration
plotMDS(dgList)

#2.6 Setting up the Model
sampleType<- rep("N", ncol(dgList)) #N=normal; T=tumour
sampleType[grep("T", colnames(dgList))] <- "T" # 'grep' is a string matching function: replaces N with T where it was 16T, 18T and 19T
sampleReplicate <- paste("S", rep(1:3, each=2), sep="")
designMat <- model.matrix(~sampleReplicate + sampleType)
designMat

#counts are modeled with Poisson distribution.
#but a Poisson distribution is parameterized by  λ , which happens to be both its mean and variance. 
#we have under- or overdispersion, depending on if the variance is smaller or larger than the mean
#One approach that addresses this issue is Negative Binomial Regression.
#Variance (=dispertion) needs to beestimated.

#2.7 Estimating Dispersions
#As disucssed, we need to estimate the dispersion parameter for our negative binomial model.
#?estimateGLMCommonDisp
dgList <- estimateGLMCommonDisp(dgList, design=designMat)
dgList <- estimateGLMTrendedDisp(dgList, design=designMat)
dgList <- estimateGLMTagwiseDisp(dgList, design=designMat)

plotBCV(dgList)

#2.8 Differential Expression
fit <- glmFit(dgList, designMat) #To fit a negative binomial generalized log-linear model
lrt <- glmLRT(fit, coef=4) #Fit a negative binomial generalized log-linear model to the read counts for each gene
#We want to see DE genes between tumor and normal, thats why coeff=4 (4th column of the design matrix)
edgeR_result <- topTags(lrt)
?topTags #Extracts the most differentially expressed genes (or sequence tags) from a test object, ranked either by p-value or by absolute log-fold-change.

edgeR_result=topTags(lrt,n=15000)$table
save(edgeR_result, file='/Users/rsonmez/Desktop/teaching/bioinformatics-core-shared-training-cruk-bioinf-sschool-24d7cad/Day3/edgeR_Result.RData') #We will need this later

#Finally, we can plot the log-fold changes of all the genes, and the highlight those that are differentially expressed.
#?decideTests
deGenes <- decideTestsDGE(lrt, p=0.001)
deGenes <- rownames(lrt)[as.logical(deGenes)]
plotSmear(lrt, de.tags=deGenes)
abline(h=c(-1, 1), col=2)
